/**********************************************************************
 *  cgilibcso.c -- libcgi portion for finger/CSO stuff
 *
 * Copyright 1994 by the Massachusetts Institute of Technology
 * For copying and distribution information, please see the file
 * <mit-copyright.h>.
 **********************************************************************/
#include "mit-copyright.h"

#define CGI_FINGER_MAX 32768

#include <errno.h>
#include <string.h>
#include <ctype.h>
#include "cgi.h"
#include "cgicso.h"

char *
cgi_next_line_in_string(string, linebuf, linebuflen)
     char *string, *linebuf;
     int linebuflen;
{
  char *ptr;

  ptr=strchr(string, '\n');

  if (ptr)
    {
      int linelen;

      linelen = ptr - string;
      if (linelen >= linebuflen) linelen = linebuflen-1;
      strncpy(linebuf, string, linelen);
      linebuf[linelen]='\0';
      return(ptr+1);
    }
  strncpy(linebuf, string, linebuflen);
  return((char *)0);
}

/* returns 1 if field found, 0 otherwise */
/* fills in valuebuf with field value on match */
int
cgi_cso_field(linebuf, fieldname, valuebuf, buflen)
     char *linebuf, *fieldname, *valuebuf;
     int buflen;
{
  char *ptr;

  ptr=strchr(linebuf, ':');
  if (!ptr) return(0);		/* not of form name: value */

  /* check if field name matches */
  if (strncmp(fieldname, ptr-strlen(fieldname), strlen(fieldname)))
    return(0);			/* doesn't match */

  /* match! */
  strncpy(valuebuf, ptr+2, buflen);
  return(1);
}

void
cgi_cso_header(query)
     char *query;
{
  /* print top of HTML doc */
  puts("Content-Type: text/html\n");
  fputs("<HEAD><TITLE>Results of directory query: \"", stdout);
  fputs(query, stdout);
  puts("\"</TITLE></HEAD>");
  fputs("<BODY><H1>Results of directory query: \"", stdout);
  fputs(query, stdout);
  puts("\"</H1>");
  return;
  /* be sure to close </BODY> */
}

void
cgi_url_hack(linebuf)
     char linebuf[];
{
  if (!strncmp(linebuf, "       url:", 11))
    {
      char urlbuf[256], *ptr1;

      strcpy(urlbuf, linebuf+12);
      /* remove trailing whitespace */
      for(ptr1=urlbuf+strlen(urlbuf)-1;
	  isspace(*ptr1) && ptr1 > urlbuf;
	  ptr1--) *ptr1 = '\0';

      printf("%11.11s <a href=\"%s\">%s</a>\n", linebuf, urlbuf, urlbuf);
    }
  else puts(linebuf);
  return;
}

int
cgi_standard_cso()
{
  cgi_form form;
  char *query, fingerbuf[CGI_FINGER_MAX], *ptr;
  char linebuf[256], namebuf[256], deptbuf[256],
  aliasbuf[256], titlebuf[256], yearbuf[256];
  int i, fieldlen, motdlen=0;
  char *csourl, *fingerhost;

  if (cgi_alloc_form(&form) ||
      cgi_parse_form(&form))
    {
      cgi_output_failure(&form, "Request was not processed due to an error.");
      cgi_free_form(&form);
      return(1);
    }
  query=cgi_value(&form, "query");

#ifdef CGI_CSO_HARDCODE
  csourl=CGI_CSO_URL;
  fingerhost=CGI_CSO_FINGERHOST;
#else
  /* make sure form has required values */
  csourl=cgi_value(&form, "csourl");
  fingerhost=cgi_value(&form, "fingerhost");
  if (!*csourl || !*fingerhost)
    {
      form.errcond=1;
      strcpy(form.errmsg, "400 Required fields missing: csourl, fingerhost");
      cgi_output_failure(&form, "Request was not processed due to an error.");
      cgi_free_form(&form);
      return(1);
    }
#endif

  /* get info from finger */
  if (finger(query, fingerhost, fingerbuf, CGI_FINGER_MAX) < 0)
    {
      form.errcond=1;
      sprintf(form.errmsg, "500 Could not finger %s@%s", query, fingerhost);
      cgi_concat_errno(form.errmsg);
      cgi_output_failure(&form, "Request was not processed due to an error.");
      cgi_free_form(&form);
      return(1);
    }

  cgi_fix_crlf(fingerbuf);

  /* go through query looking for "There were NN matches" */
  ptr=fingerbuf; 
  while ((ptr=cgi_next_line_in_string(ptr, linebuf, 256)) != (char *)0)
    {
      if (!strncmp(linebuf, "There were ", 11)) break;
      motdlen = ptr - fingerbuf;
    }

  /* not found -- just print out whatever finger returned */
  if (!ptr)
    {
      cgi_cso_header(query);
      puts("<PRE>");
      /* URL hack */
      ptr=fingerbuf; 
      while ((ptr=cgi_next_line_in_string(ptr, linebuf, 256)) != (char *)0)
	cgi_url_hack(linebuf);
      puts("</PRE>");
      puts("</BODY>");
      return(0);
    }

  /* found -- print nice HTML */
  deptbuf[0] = '\0';		/* no dept. found yet */
  cgi_cso_header(query);
  puts(linebuf);
  puts("<PRE>");
  while ((ptr=cgi_next_line_in_string(ptr, linebuf, 256)) != (char *)0)
    {
      if (cgi_cso_field(linebuf, "department", deptbuf, 256)) continue;
      if (cgi_cso_field(linebuf, "name", namebuf, 256)) continue;
      if (cgi_cso_field(linebuf, "title", titlebuf, 256)) continue;
      if (cgi_cso_field(linebuf, "year", yearbuf, 256)) continue;
      if (cgi_cso_field(linebuf, "alias", aliasbuf, 256))
	{
	  /* put hyperlink to direct CSO GET query */
	  fputs("<A HREF=\"", stdout);
	  fputs(csourl, stdout);
	  putchar((int)'?');
	  fputs(aliasbuf, stdout);
	  fputs("\">", stdout);
	  fputs(namebuf, stdout);

	  /* put periods up to next field */
	  fputs("</A>...", stdout);
	  fieldlen = 24-strlen(namebuf);
	  for(i=0; i<fieldlen; i++) putchar((int) '.');

	  /* put "dept, year n", "dept", or "Year n" as appropriate */
	  /* fall back on "title" if available */
	  if (*deptbuf)
	    {
	      if (*yearbuf)
		{
		  fputs(deptbuf, stdout);
		  fputs(", year ", stdout);
		  puts(yearbuf);
		}
	      else puts(deptbuf);
	    }
	  else
	    if (*yearbuf)
	      {
		fputs("Year ", stdout);
		puts(yearbuf);
	      }
	    else
	      if (*titlebuf) puts(titlebuf);
	      else puts("");
	  deptbuf[0] = titlebuf[0] = yearbuf[0] = '\0';
	}
    }

  puts("\n<HR>\n");
  /* print motd that precedes data */
  fwrite(fingerbuf, sizeof(char), motdlen, stdout);
  puts("</PRE></BODY>");
  return(0);
}
